window.Gleam.initSearch([{"doc":"gary","title":"gary","content":"# gary\n\nA Gleam interface to Erlang&#39;s `array` module, for sparse, efficiently-implemented functional arrays. 🐝\n\n[![Package Version](https://img.shields.io/hexpm/v/gary)](https://hex.pm/packages/gary)\n[![Hex Docs](https://img.shields.io/badge/hex-docs-ffaff3)](https://hexdocs.pm/gary/)\n\n## When should I use `gary`?  When are other data types better?\n\nErlang arrays are $O(\\log n)$ for both lookups and updates.  If you don&#39;t remember big-O notation, for the purposes of this explanation that essentially means &quot;faster than $O(n)$ (linear time), but slower than $O(1)$ (constant time)&quot;.\n\nTuples are $O(1)$ for lookups, but $O(n)$ for updates -- you can lookup a tuple by its index without iterating over the tuple, but the entire tuple has to be copied to a new location in memory to update one of its values.  Tuples are faster when you&#39;re doing repeated random lookups on an unchanging set of values, but lose out to arrays when random updates are required.  \n\n[`glearray`](https://hex.pm/packages/glearray)&#39;s version of arrays are represented as a single tuple, so if you&#39;re solely doing lookups and don&#39;t need the sparse representation or dynamically resizeable arrays, `glearray` will likely be faster than `gary` .\n\nLists are $O(n)$ for both lookups and updates anywhere but the head of the list, since you need to iterate over the pairs of the linked list to find an arbitrary index within it.  Lists will be faster that arrays if you can ensure that you only ever operate at the head of the list, like in most traditional recursive algorithms.\n\n## Example\n\n```sh\ngleam add gary\n```\n\n```gleam\nimport gary/array\n\npub fn main() {\n  list.range(1, 10)\n      |&gt; array.from_list(default: -99)\n      |&gt; array.drop(at: 5)\n      |&gt; result.map(array.map(_, fn(_, v) { 2 * v }))\n      |&gt; result.map(array.to_list)\n  // = Ok([2, 4, 6, 8, 10, -198, 14, 16, 18, 20])\n}\n```\n\nFurther documentation can be found at &lt;https://hexdocs.pm/gary&gt;.\n\n## Development\n\n```sh\ngleam run   # Run the project\ngleam test  # Run the tests\ngleam shell # Run an Erlang shell\n```\n","url":"index.html"},{"doc":"gary","title":"ErlangArray","content":"pub type ErlangArray(a)\n\n\nSynonyms:\ngary.ErlangArray\ngary ErlangArray","url":"gary.html#ErlangArray"},{"doc":"gary","title":"main","content":"pub fn main() -&gt; Int\n\nSynonyms:\ngary.main\ngary main","url":"gary.html#main"},{"doc":"gary","title":"gary","content":" This module only contains the `ErlangArray` type, as used by the functions in `gary/array`.","url":"gary.html"},{"doc":"gary/array","title":"ArrayError","content":"pub type ArrayError {\n  IndexOutOfRange\n  BadSize\n}\n Types representing invalid situations that would otherwise cause the function call to fail with\n a `badarg` exception in Erlang.\nIndexOutOfRange\n attempted to access a negative index, or an index beyond the size of a fixed-size array\n\nBadSize\n tried to make an array with a negative size\n\nSynonyms:\ngary/array.ArrayError\ngary/array ArrayError","url":"gary/array.html#ArrayError"},{"doc":"gary/array","title":"create","content":"pub fn create(default default: a) -&gt; ErlangArray(a)\n Creates a new, extensible sparse array with a default value of `default` for any indices\n without an assigned value.\nSynonyms:\ngary/array.create\ngary/array create","url":"gary/array.html#create"},{"doc":"gary/array","title":"create_fixed_size","content":"pub fn create_fixed_size(\n  size size: Int,\n  default default: a,\n) -&gt; Result(ErlangArray(a), ArrayError)\n Creates a new, fixed-size sparse array of size `size` with a default value of `default` \n for any indices without an assigned value.  Returns a `BadSize` error if `size` is negative.\nSynonyms:\ngary/array.create_fixed_size\ngary/array create_fixed_size","url":"gary/array.html#create_fixed_size"},{"doc":"gary/array","title":"drop","content":"pub fn drop(\n  from array: ErlangArray(a),\n  at index: Int,\n) -&gt; Result(ErlangArray(a), ArrayError)\n Returns a new array where the value at index `index` has been set to the array&#39;s default.  \n Returns `Error(IndexOutOfRange)` if the index is negative, or if it exceeds the \n size of a fixed-size array.\nSynonyms:\ngary/array.drop\ngary/array drop","url":"gary/array.html#drop"},{"doc":"gary/array","title":"fold","content":"pub fn fold(\n  over array: ErlangArray(a),\n  from initial: b,\n  with function: fn(Int, a, b) -&gt; b,\n) -&gt; b\n Reduces the array values into a single value by calling `function` on each array value,\n including default values, from left to right (beginning at index 0).\nSynonyms:\ngary/array.fold\ngary/array fold","url":"gary/array.html#fold"},{"doc":"gary/array","title":"fold_right","content":"pub fn fold_right(\n  over array: ErlangArray(a),\n  from initial: b,\n  with function: fn(Int, a, b) -&gt; b,\n) -&gt; b\n Reduces the array values into a single value by calling `function` on each array value,\n including default values, from right to left (ending at index 0).\nSynonyms:\ngary/array.fold_right\ngary/array fold_right","url":"gary/array.html#fold_right"},{"doc":"gary/array","title":"from_dict","content":"pub fn from_dict(\n  from dict: Dict(Int, a),\n  default default: a,\n) -&gt; Result(ErlangArray(a), ArrayError)\n Converts a `dict` with integer keys representing array indices to an extensible sparse array.  \n Returns `IndexOutOfRange` if any of the keys are negative.\n \n \n [#(23, &quot;Michael Jordan&quot;), #(33, &quot;Larry Bird&quot;), #(73, &quot;Dennis Rodman&quot;)]\n |&gt; dict.from_list\n |&gt; array.from_dict(default: &quot;Unknown&quot;)\n |&gt; result.try(array.get(from: _, at: 33))\n // -&gt; Ok(&quot;Larry Bird&quot;)\n \n \nSynonyms:\ngary/array.from_dict\ngary/array from_dict","url":"gary/array.html#from_dict"},{"doc":"gary/array","title":"from_list","content":"pub fn from_list(\n  from list: List(a),\n  default default: a,\n) -&gt; ErlangArray(a)\n Converts a list to an extensible sparse array, assigning list members to indices \n starting at an index of 0.\n \n \n [&quot;trains&quot;, &quot;planes&quot;, &quot;automobiles&quot;]\n |&gt; list.map(option.Some)\n |&gt; array.from_list(default: option.None)\n |&gt; array.get(at: 1)\n // -&gt; Ok(Some(&quot;planes&quot;))\n \n \nSynonyms:\ngary/array.from_list\ngary/array from_list","url":"gary/array.html#from_list"},{"doc":"gary/array","title":"get","content":"pub fn get(\n  from array: ErlangArray(a),\n  at index: Int,\n) -&gt; Result(a, ArrayError)\n Returns the value at index `index`.\n Returns `Error(IndexOutOfRange)` if the index is negative, or if it exceeds the \n size of a fixed-size array.\nSynonyms:\ngary/array.get\ngary/array get","url":"gary/array.html#get"},{"doc":"gary/array","title":"get_count","content":"pub fn get_count(array: ErlangArray(a)) -&gt; Int\n Returns the number of defined elements in the array.\nSynonyms:\ngary/array.get_count\ngary/array get_count","url":"gary/array.html#get_count"},{"doc":"gary/array","title":"get_default","content":"pub fn get_default(array: ErlangArray(a)) -&gt; a\n Returns the array&#39;s default for unset values.\nSynonyms:\ngary/array.get_default\ngary/array get_default","url":"gary/array.html#get_default"},{"doc":"gary/array","title":"get_size","content":"pub fn get_size(array: ErlangArray(a)) -&gt; Int\n Returns the size of the array.\n For fixed-size arrays, this is the defined size of the array.\n For extensible arrays, if the greatest index with a defined value is `n`, the size is `n + 1`.\nSynonyms:\ngary/array.get_size\ngary/array get_size","url":"gary/array.html#get_size"},{"doc":"gary/array","title":"is_fixed_size","content":"pub fn is_fixed_size(array: ErlangArray(a)) -&gt; Bool\n Returns `True` if the array is fixed-size, and `False` if it&#39;s extensible.\nSynonyms:\ngary/array.is_fixed_size\ngary/array is_fixed_size","url":"gary/array.html#is_fixed_size"},{"doc":"gary/array","title":"make_extensible","content":"pub fn make_extensible(array: ErlangArray(a)) -&gt; ErlangArray(a)\n Returns an extensible version of the array.\nSynonyms:\ngary/array.make_extensible\ngary/array make_extensible","url":"gary/array.html#make_extensible"},{"doc":"gary/array","title":"make_fixed","content":"pub fn make_fixed(array: ErlangArray(a)) -&gt; ErlangArray(a)\n Returns a fixed-size version of the array, sized to fit the set values.\nSynonyms:\ngary/array.make_fixed\ngary/array make_fixed","url":"gary/array.html#make_fixed"},{"doc":"gary/array","title":"map","content":"pub fn map(\n  over array: ErlangArray(a),\n  with function: fn(Int, a) -&gt; b,\n) -&gt; ErlangArray(b)\n Returns a new array that applies `function` to each array value, including default values\n within the current size of the array (see `get_size` for what this means for extensible arrays).\nSynonyms:\ngary/array.map\ngary/array map","url":"gary/array.html#map"},{"doc":"gary/array","title":"resize_to_fit","content":"pub fn resize_to_fit(array: ErlangArray(a)) -&gt; ErlangArray(a)\n If `array` is fixed-size, returns an array with its bounds resized to \n exactly fit the set values. Has no effect on extensible arrays; just returns\n the original array.\nSynonyms:\ngary/array.resize_to_fit\ngary/array resize_to_fit","url":"gary/array.html#resize_to_fit"},{"doc":"gary/array","title":"set","content":"pub fn set(\n  into array: ErlangArray(a),\n  at index: Int,\n  put item: a,\n) -&gt; Result(ErlangArray(a), ArrayError)\n Returns a new array with `item` assigned to index `index`. \n Returns `Error(IndexOutOfRange)` if the index is negative,\n or if it exceeds the size of a fixed-size array.\nSynonyms:\ngary/array.set\ngary/array set","url":"gary/array.html#set"},{"doc":"gary/array","title":"set_size","content":"pub fn set_size(\n  array: ErlangArray(a),\n  size: Int,\n) -&gt; Result(ErlangArray(a), ArrayError)\n Returns a fixed-size version of the array with its size set to `size`.\n Returns a `BadSize` error if `size` is negative.\nSynonyms:\ngary/array.set_size\ngary/array set_size","url":"gary/array.html#set_size"},{"doc":"gary/array","title":"sparse_fold","content":"pub fn sparse_fold(\n  over array: ErlangArray(a),\n  from initial: b,\n  with function: fn(Int, a, b) -&gt; b,\n) -&gt; b\n Reduces the array values into a single value by calling `function` on each array value,\n excluding default values, from left to right (beginning at index 0).\nSynonyms:\ngary/array.sparse_fold\ngary/array sparse_fold","url":"gary/array.html#sparse_fold"},{"doc":"gary/array","title":"sparse_fold_right","content":"pub fn sparse_fold_right(\n  over array: ErlangArray(a),\n  from initial: b,\n  with function: fn(Int, a, b) -&gt; b,\n) -&gt; b\n Reduces the array values into a single value by calling `function` on each array value,\n excluding default values, from right to left (ending at index 0).\nSynonyms:\ngary/array.sparse_fold_right\ngary/array sparse_fold_right","url":"gary/array.html#sparse_fold_right"},{"doc":"gary/array","title":"sparse_map","content":"pub fn sparse_map(\n  over array: ErlangArray(a),\n  with function: fn(Int, a) -&gt; b,\n) -&gt; ErlangArray(b)\n Returns a new array that applies `function` to each array value, excluding default values.\nSynonyms:\ngary/array.sparse_map\ngary/array sparse_map","url":"gary/array.html#sparse_map"},{"doc":"gary/array","title":"to_dict","content":"pub fn to_dict(array: ErlangArray(a)) -&gt; Dict(Int, a)\n Converts the array to a `dict` with integer keys, including the default value for any keys \n between 0 and the maximum that don&#39;t have values.\nSynonyms:\ngary/array.to_dict\ngary/array to_dict","url":"gary/array.html#to_dict"},{"doc":"gary/array","title":"to_dict_without_defaults","content":"pub fn to_dict_without_defaults(\n  array: ErlangArray(a),\n) -&gt; Dict(Int, a)\n Converts the array to a `dict` with integer keys, omitting default values.\nSynonyms:\ngary/array.to_dict_without_defaults\ngary/array to_dict_without_defaults","url":"gary/array.html#to_dict_without_defaults"},{"doc":"gary/array","title":"to_keylist","content":"pub fn to_keylist(array: ErlangArray(a)) -&gt; List(#(Int, a))\n Converts the array to a list of 2-tuples with the key as the first element\n (an Erlang `orddict`, AKA a keyword list or association list).\nSynonyms:\ngary/array.to_keylist\ngary/array to_keylist","url":"gary/array.html#to_keylist"},{"doc":"gary/array","title":"to_keylist_without_defaults","content":"pub fn to_keylist_without_defaults(\n  array: ErlangArray(a),\n) -&gt; List(#(Int, a))\n Converts the array to a list of 2-tuples with the key as the first element\n (an Erlang `orddict`, AKA a keyword list or association list),\n omitting default values.\nSynonyms:\ngary/array.to_keylist_without_defaults\ngary/array to_keylist_without_defaults","url":"gary/array.html#to_keylist_without_defaults"},{"doc":"gary/array","title":"to_list","content":"pub fn to_list(array: ErlangArray(a)) -&gt; List(a)\n Converts the array&#39;s values to a list, in order from index 0 to the maximum,\n including the default value for any keys between 0 and\n the maximum that don&#39;t have values.\n \n \n [#(0, &quot;nada&quot;), #(2, &quot;company&quot;), #(3, &quot;a crowd&quot;), #(5, &quot;a carful&quot;)]\n |&gt; dict.from_list\n |&gt; array.from_dict(default: &quot;something else&quot;)\n |&gt; result.map(array.to_list)\n // -&gt; Ok([&quot;nada&quot;, &quot;something else&quot;, &quot;company&quot;, &quot;a crowd&quot;, &quot;something else&quot;, &quot;a carful&quot;])\n \n \nSynonyms:\ngary/array.to_list\ngary/array to_list","url":"gary/array.html#to_list"},{"doc":"gary/array","title":"to_list_without_defaults","content":"pub fn to_list_without_defaults(array: ErlangArray(a)) -&gt; List(a)\n Converts the array&#39;s values to a list, in order from lowest to highest index,\n omitting default values.\n \n \n [#(0, &quot;nada&quot;), #(2, &quot;company&quot;), #(3, &quot;a crowd&quot;), #(5, &quot;a carful&quot;)]\n |&gt; dict.from_list\n |&gt; array.from_dict(default: &quot;something else&quot;)\n |&gt; result.map(array.to_list)\n // -&gt; Ok([&quot;nada&quot;, &quot;company&quot;, &quot;a crowd&quot;, &quot;a carful&quot;])\n \n \nSynonyms:\ngary/array.to_list_without_defaults\ngary/array to_list_without_defaults","url":"gary/array.html#to_list_without_defaults"},{"doc":"gary/array","title":"gary/array","content":" Functional, extendible arrays, provided by Erlang&#39;s \n [`array`](https://www.erlang.org/doc/man/array) module. Arrays can have fixed size,\n or can grow automatically as needed (&quot;extensible&quot;). A default value is used for entries\n that have not been explicitly set.\n \n Erlang arrays are 0-indexed.\n The maximum index for a fixed-size array of size `n` is `n - 1`.\n There is no maximum index for a extensible array; the array resizes to fit as necessary.\n Negative indices aren&#39;t supported.\n \n Erlang doesn&#39;t distinguish between an unset entry and a entry that&#39;s been explicitly set \n to the array&#39;s defined default value, which can cause issues when using functions that\n drop unset entries like `sparse_map` or `sparse_to_list`.  Use a type like `Option(t)` to \n ensure that you can&#39;t accidentally cause this kind of collision.\n \n See the Erlang docs for more information and caveats.\n ","url":"gary/array.html"}]);